@@ -25,6 +25,7 @@ NotImplementedError
 """
 
 import logging
+import multiprocessing as mp
 import os
 import shutil
 import time
@@ -38,6 +39,7 @@ import metator.partition as mtp
 import metator.scaffold as mts
 import metator.validation as mtv
 from docopt import docopt
+from functools import partial
 from metator.log import logger
 from metator.version import __version__
 from os.path import exists, dirname, join
@@ -611,9 +613,8 @@ class Pipeline(AbstractCommand):
     usage:
         pipeline --assembly=FILE [--forward=STR] [--reverse=STR]
         [--algorithm=louvain] [--aligner=bowtie2] [--aligner-mode=normal]
-        [--cluster-matrix] [--contigs=FILE] [--depth=FILE] [--edge=0]
-        [--enzyme=STR] [--force] [--iterations=100] [--rec-iter=10]
-        [--junctions=NNNNN] [--network=FILE] [--no-clean-up]
+        [--cluster-matrix] [--depth=FILE] [--edge=0] [--enzyme=STR] [--force]
+        [--iterations=100] [--rec-iter=10] [--junctions=NNNNN] [--no-clean-up]
         [--normalization=empirical_hit] [--outdir=DIR] [--overlap=80]
         [--rec-overlap=90]  [--min-quality=30] [--res-param=1.0]
         [--size=500000] [--start=fastq] [--threads=1] [--tmpdir=DIR]
@@ -639,10 +640,6 @@ class Pipeline(AbstractCommand):
                                 "bowtie2". [Default: bowtie2]
         -B, --aligner-mode=STR  Mode of alignment from hicstuff. Either normal,
                                 iterative or cutsite. [Default: normal]
-        -c, --contigs=FILE      The path to the file containing the data ofthe
-                                contigs (ID, Name, Length, GC content, Hit,
-                                Coverage, Restriction site). Required if start
-                                network.
         -C, --cluster-matrix    If enabled, save the clustering matrix.
         -d, --depth=FILE        The depth.txt file from the shotgun reads used
                                 to made the assembly computed by
@@ -662,9 +659,6 @@ class Pipeline(AbstractCommand):
                                 recursive step. [Default: 10]
         -J, --junctions=STR     Sequences to use as junction between contigs.
                                 [Default: NNNNN]
-        -n, --network=FILE      Path to the file containing the network
-                                information from the meta HiC experiment compute
-                                in network function previously.
         -N, --no-clean-up       Do not remove temporary files.
         -m, --normalization=STR If None, do not normalized the count of a
                                 contact by the geometric mean of the coverage of
@@ -686,12 +680,14 @@ class Pipeline(AbstractCommand):
                                 algorithm. [Default: 1.0]
         -s, --size=INT          Threshold size to keep bins in base pair.
                                 [Default: 500000]
-        -S, --start=STR         Start stage of the pipeline. Either fastq, bam
-                                pair, or network. [Default: fastq]
+        -S, --start=STR         Start stage of the pipeline. Either fastq, bam,
+                                or pair. [Default: fastq]
         -t, --threads=INT       Number of parallel threads allocated for the
                                 alignement. [Default: 1]
-        -T, --tmpdir=DIR       Temporary directory. Default to current
+        -T, --tmpdir=DIR        Temporary directory. Default to current
                                 directory. [Default: ./tmp]
+        -v, --scaffold          If enables, it will sacffold the genomes at the
+                                end.
     """
 
     def execute(self):
@@ -831,19 +827,20 @@ class Pipeline(AbstractCommand):
                 raise ValueError
 
         # Sanity check for scaffolding
-        scaffold_fasta_dir = join(self.args["--outdir"], "scaffold_bin")
-        if not exists(scaffold_fasta_dir):
-            os.makedirs(scaffold_fasta_dir)
-        else:
-            if self.args["--force"]:
-                shutil.rmtree(scaffold_fasta_dir)
+        if self.args["--scaffold"]:
+            scaffold_fasta_dir = join(self.args["--outdir"], "scaffold_bin")
+            if not exists(scaffold_fasta_dir):
                 os.makedirs(scaffold_fasta_dir)
             else:
-                logger.error(
-                    "%s already existed. Remove directory or use -F argument to overwrite it.",
-                    scaffold_fasta_dir,
-                )
-                raise ValueError
+                if self.args["--force"]:
+                    shutil.rmtree(scaffold_fasta_dir)
+                    os.makedirs(scaffold_fasta_dir)
+                else:
+                    logger.error(
+                        "%s already existed. Remove directory or use -F argument to overwrite it.",
+                        scaffold_fasta_dir,
+                    )
+                    raise ValueError
 
         # Manage start point.
         if self.args["--start"] == "fastq":
@@ -852,12 +849,8 @@ class Pipeline(AbstractCommand):
             start = 2
         elif self.args["--start"] == "pair":
             start = 3
-        elif self.args["--start"] == "network":
-            start = 4
         else:
-            logger.error(
-                "Start argument should be 'fastq', 'bam', 'pair' or 'network'."
-            )
+            logger.error("Start argument should be 'fastq', 'bam', or 'pair'.")
             raise ValueError
 
         # Check if forward and reverse reads are given for fastq and bam start.
@@ -911,50 +904,46 @@ class Pipeline(AbstractCommand):
             fasta = mio.retrieve_fasta(index, self.args["--aligner"], tmp_dir)
 
         # Run the whole workflow
-        if start <= 3:
-            if start <= 2:
-                # Align pair-end reads with bowtie2
-                alignment_files, contig_data, hit_data = mta.get_contact_pairs(
-                    self.args["--forward"],
-                    self.args["--reverse"],
-                    index,
-                    fasta,
-                    self.args["--aligner"],
-                    self.args["--aligner-mode"],
-                    min_qual,
-                    self.args["--start"],
-                    self.args["--depth"],
-                    self.args["--enzyme"],
-                    self.args["--outdir"],
-                    tmp_dir,
-                    self.args["--threads"],
-                )
-            else:
-                alignment_files = self.args["--forward"].split(",")
-                nb_alignment = len(alignment_files)
-                contig_data, hit_data = mtn.create_contig_data(
-                    fasta,
-                    nb_alignment,
-                    self.args["--depth"],
-                    self.args["--enzyme"],
-                )
-            # Build the network
-            network_file, contigs_data_file = mtn.alignment_to_contacts(
-                alignment_files,
-                contig_data,
-                edge,
-                hit_data,
+        if start <= 2:
+            # Align pair-end reads with bowtie2
+            alignment_files, contig_data, hit_data = mta.get_contact_pairs(
+                self.args["--forward"],
+                self.args["--reverse"],
+                index,
+                fasta,
+                self.args["--aligner"],
+                self.args["--aligner-mode"],
+                min_qual,
+                self.args["--start"],
+                self.args["--depth"],
+                self.args["--enzyme"],
                 self.args["--outdir"],
-                "network.txt",
-                "contig_data_network.txt",
                 tmp_dir,
                 self.args["--threads"],
-                self.args["--normalization"],
-                False,
             )
         else:
-            contigs_data_file = self.args["--contigs"]
-            network_file = self.args["--network"]
+            alignment_files = self.args["--forward"].split(",")
+            nb_alignment = len(alignment_files)
+            contig_data, hit_data = mtn.create_contig_data(
+                fasta,
+                nb_alignment,
+                self.args["--depth"],
+                self.args["--enzyme"],
+            )
+        # Build the network
+        network_file, contigs_data_file = mtn.alignment_to_contacts(
+            alignment_files,
+            contig_data,
+            edge,
+            hit_data,
+            self.args["--outdir"],
+            "network.txt",
+            "contig_data_network.txt",
+            tmp_dir,
+            self.args["--threads"],
+            self.args["--normalization"],
+            False,
+        )
 
         # Partition the network
         clustering_matrix_partition_file, contigs_data_file = mtp.partition(
@@ -1022,21 +1011,19 @@ class Pipeline(AbstractCommand):
         os.remove(contig_data_partition_file)
 
         # Launch the scaffold
-        bin_summary = mio.read_bin_summary(
-            join(self.args["--outdir"], "bin_summary.txt")
-        )
-        for bin_name in bin_summary.index:
-            mts.get_scaffolds(
-                bin_name,
-                join(final_fasta_dir, f"{bin_name}.fa"),
-                alignment_files,
-                join(scaffold_fasta_dir, f"{bin_name}_scaffolded.fa"),
-                join(scaffold_fasta_dir, f"{bin_name}_info_frags.txt"),
-                threshold=0.05,
-                threads=int(self.args["--threads"]),
-                window_size=5000,
+        if self.args["--scaffold"]:
+            bin_summary = mio.read_bin_summary(
+                join(self.args["--outdir"], "bin_summary.txt")
+            )
+            task = partial(
+                mts.parallel_scaffold,
+                final_fasta_dir=final_fasta_dir,
+                alignment_files=alignment_files,
+                scaffold_fasta_dir=scaffold_fasta_dir,
                 junctions=self.args["--junctions"],
             )
+            pool = mp.Pool(processes=int(self.args["--threads"]))
+            pool.map(task, bin_summary.index)
 
         # Delete pyfastx index:
         os.remove(fasta + ".fxi")
@@ -1204,10 +1191,13 @@ class Contactmap(AbstractCommand):
     using the output.
 
     usage:
-        contactmap --assembly=FILE --contig-data=FILE --enzyme=STR --name=STR --pairs=FILE
+        contactmap --assembly=FILE --contig-data=FILE --enzyme=STR --name=STR
         [--filter] [--force] [--mat-fmt=graal] [--min-size=5000]
         [--no-clean-up] [--object=final_bin] [--outdir=DIR] [--pcr-dup]
-        [--tmpdir=DIR] [--threads=1]
+        [--tmpdir=DIR] [--threads=1] <pairsfile>...
+
+    argument:
+        pairsfile               File(s) containing pairs information.
 
     options:
         -a, --assembly=FILE     Path to the fasta file containing the contigs of
@@ -1221,8 +1211,6 @@ class Contactmap(AbstractCommand):
         -n, --name=STR          Name of the metator or its numerical ID for the
                                 core bin, overlapping bin or recursive bin
                                 objects. Example: "NODE_1", "MetaTOR_1_0" or 8.
-        -p, --pairs=FILE        Path of the ".pairs" file. If more than one is
-                                given, files should be separated by a comma.
         -D, --pcr-dup,          Filter out PCR duplicates based on read
                                 positions.
         -f, --filter            Filter out spurious 3C events (loops and uncuts)
@@ -1269,7 +1257,7 @@ class Contactmap(AbstractCommand):
             self.args["--contig-data"],
             self.args["--enzyme"],
             self.args["--name"],
-            self.args["--pairs"],
+            self.args["<pairsfile>"],
             self.args["--outdir"],
             tmp_dir,
             self.args["--filter"],
@@ -1298,7 +1286,7 @@ class Scaffold(AbstractCommand):
 
     usage:
         scaffold --bin-name=STR --input-fasta=FILE [--junctions=NNNNN]
-        [--out-fasta=FILE] [--out-frags=FILE] [--threads=1] [--threshold=0.1]
+        [--out-fasta=FILE] [--out-frags=FILE] [--threads=1] [--threshold=0.05]
         [--window-size=5000] <pairsfile>...
 
     arguments:
@@ -1317,7 +1305,7 @@ class Scaffold(AbstractCommand):
         -t, --threads=INT       Numbers of threads to allocate if pairs need to
                                 be sorted. [Default: 1]
         -T, --threshold=FLOAT   Threshold score to consider an association
-                                between two contigs. [Default 0.1]
+                                between two contigs. [Default: 0.05]
         -w, --window-size=INT   Size of the window in base pair to use as edge
                                 window to scaffold. [Default: 5000]
     """
