@@ -9,17 +9,21 @@ distance to group together bins relatively closed (to avaoid to split genomes in
 different bins).
 
 Core functions to partition the network are:
+    - algo_partition
     - build_clustering_matrix
     - defined_overlapping_bins
     - detect_core_bins
     - generate_fasta
     - get_distances_splitmat
     - get_hamming_distance
-    - leiden_partition_java
-    - louvain_partition_cpp
+    - leiden_iterations_java
+    - louvain_iterations_cpp
     - partition
     - remove_isolates
     - update_contigs_data
+
+Deprecated Spinglass functions:
+    - spinglass_partition
 """
 
 import metator.io as mio
@@ -28,6 +32,8 @@ import numpy as np
 import os
 import pandas as pd
 import subprocess as sp
+
+# from cdlib import algorithms
 from functools import partial
 from metator.log import logger
 from os.path import join
@@ -35,6 +41,77 @@ from scipy import sparse
 from sklearn import metrics
 
 
+def algo_partition(
+    algorithm="louvain",
+    network_file=None,
+    network=None,
+    iterations=10,
+    resolution_parameter=1.0,
+    tmpdir=".",
+    spin=2,
+):
+    """Function to partition the network depednding on the used algorithm.
+
+    Parameters:
+    -----------
+    algorithm : str
+        Algorithm to use to partition network. [Default: louvain]
+    network_file : str
+        Path to the network computed previously. The file is 3 columns table
+        separated by a tabulation with the id of the first contigs the id of the
+        second one and the weights of the edge normalized or not. Mandatory if
+        louvain or leiden algorithm. [Default: None]
+    network : networkx.classes.graph.Graph
+        Network of interaction of a contaminated bins. Mandatory if spinglass
+        algorithm. [Default: None]
+    iterations : int
+        Number of iterations of the algorithm of Leiden or Louvain.
+        [Default: 10]
+    resolution_parameter : float
+        Resolution parameter for Leiden clustering. [Default: 1.0]
+    tmp_dir : str
+        Path to the temporary directory. [Default: current directory]
+    spin : int
+        Deprecated. Number of final cluster if spinglass algorithm chosen.
+        [Default: 2]
+
+    Returns:
+    --------
+    dict:
+        Dictionnary with the id of the contig as key and the list of the results
+        of each iterations separated by a semicolon as values.
+    """
+    # Launch the write partition algorithm
+    if algorithm == "leiden":
+        LEIDEN_PATH = os.environ["LEIDEN_PATH"]
+        output_partition = leiden_iterations_java(
+            network_file,
+            iterations,
+            resolution_parameter,
+            tmpdir,
+            LEIDEN_PATH,
+        )
+    elif algorithm == "louvain":
+        LOUVAIN_PATH = os.environ["LOUVAIN_PATH"]
+        output_partition = louvain_iterations_cpp(
+            network_file,
+            iterations,
+            tmpdir,
+            LOUVAIN_PATH,
+        )
+    # elif algorithm == "spinglass":
+    #     output_partition = spinglass_partition(
+    #         network,
+    #         spins=spin,
+    #     )
+    else:
+        logger.error(
+            'algorithm should be either "louvain", "leiden", or "spinglass"'
+        )
+        raise ValueError
+    return output_partition
+
+
 def build_clustering_matrix(core_bins_contigs, hamming_distance, N):
     """Function to return the clustering matrix in sparse format.
 
@@ -76,15 +153,17 @@ def build_clustering_matrix(core_bins_contigs, hamming_distance, N):
     )
     # Compute the clustering matrix on only the upper triangle of the hamming
     # distance matrix as it's symmetric to reduce memory usage.
-    hamming_distance = sparse.triu(hamming_distance, k=1)
-    clustering_matrix = transition_matrix.T.dot(hamming_distance).dot(
-        transition_matrix
-    )
-    return ((clustering_matrix + clustering_matrix) / 2).tocoo()
+    hamming_distance = sparse.triu(hamming_distance, k=0)
+    M = transition_matrix.T.dot(hamming_distance).dot(transition_matrix)
+    M = (M + M.T).tocsr()
+    M[M > 1] = 1
+    return sparse.triu(M.tocoo(), k=0)
 
 
 def defined_overlapping_bins(
-    overlap, hamming_distance, core_bins_contigs, core_bins_iterations
+    overlap,
+    hamming_distance,
+    core_bins_contigs,
 ):
     """This function extract the overlapped bins
 
@@ -106,8 +185,6 @@ def defined_overlapping_bins(
     core_bins_contigs : dict
         Dictionnary which has as keys the core bins id and as value the id of
         the contigs of the core bin.
-    core_bins_iteration : pandas.core.frame.DataFrame
-        Table with the id of the core bin and their values for each iterations.
 
     Returns:
     --------
@@ -139,9 +216,7 @@ def defined_overlapping_bins(
             overlapping_bins[oc_id + 1] += core_bin_contigs
         cc_id += 1
 
-    logger.info(
-        "{0} overlapping bins were found.".format(len(overlapping_bins))
-    )
+    logger.info(f"{len(overlapping_bins)} overlapping bins were found.")
 
     return overlapping_bins
 
@@ -201,7 +276,7 @@ def detect_core_bins(output_partition, iterations):
     # Transform the array in a dataframe
     core_bins_iterations = pd.DataFrame(core_bins_iterations)
 
-    logger.info("{0} core bins were found.\n".format(len(core_bins)))
+    logger.info(f"{len(core_bins)} core bins were found.")
 
     return core_bins_contigs, core_bins_iterations
 
@@ -240,9 +315,9 @@ def generate_fasta(
         list_contigs_id = overlapping_bins[bin_id]
         list_contigs_name = []
         # Test if the bin is bigger than the size threshold given.
-        length_bin = contigs_data.loc[
-            list_contigs_id[0] - 1, "Overlapping_bin_size"
-        ]
+        length_bin = int(
+            contigs_data.loc[list_contigs_id[0] - 1, "Overlapping_bin_size"]
+        )
         if length_bin >= size:
             nb_bins += 1
             length_bins += length_bin
@@ -251,9 +326,9 @@ def generate_fasta(
                     contigs_data.loc[contig_id - 1, "Name"]
                 )
             # Define the output file.
-            output_file = join(output_dir, "MetaTOR_{0}_0.fa".format(bin_id))
+            output_file = join(output_dir, f"MetaTOR_{bin_id:05d}_{0:05d}.fa")
             # Create the fasta file.
-            contigs_file = join(tmpdir, "MetaTOR_{0}_0.txt".format(bin_id))
+            contigs_file = join(tmpdir, f"MetaTOR_{bin_id:05d}_{0:05d}.txt")
             with open(contigs_file, "w") as f:
                 for contig_name in list_contigs_name:
                     f.write("%s\n" % contig_name)
@@ -262,11 +337,9 @@ def generate_fasta(
             )
             process = sp.Popen(cmd, shell=True)
             process.communicate()
-    logger.info("{0} bins have been extracted".format(nb_bins))
+    logger.info(f"{nb_bins} bins have been extracted")
     logger.info(
-        "Total size of the extracted bins: {0}Mb".format(
-            round(length_bins / 10 ** 6, 3)
-        )
+        f"Total size of the extracted bins: {round(length_bins / 10**6, 3)}Mb"
     )
 
 
@@ -298,19 +371,14 @@ def get_distances_splitmat(bins, core_bins_iterations):
     return x
 
 
-def get_hamming_distance(core_bins_iterations, n_iter, threads):
+def get_hamming_distance(core_bins_iterations, threads):
     """Generate matrix of Hamming distances between all pairs of core bins.
 
     Parameters:
     -----------
     core_bins_iterations : pandas.core.frame.DataFrame
-        Table with the id of the core bin and their values for each iterations.
-    core_bins : dict
-        Dictionnary which has as keys the values of the iterations from Louvain
-        or Leiden separated by a semicolon and as value the id of the contigs of
-        the core bin.
-    n_iter : int
-        Number of iterations made previously.
+        Table with the id of the core bin as index and their values for each
+        iterations.
     threads : int
         Number of cores to parallelize computation.
 
@@ -342,7 +410,7 @@ def get_hamming_distance(core_bins_iterations, n_iter, threads):
 def leiden_iterations_java(
     network_file, iterations, resolution_parameter, tmp_dir, leiden_path
 ):
-    """Use the java implementation of Leiden to prtition the network.
+    """Use the java implementation of Leiden to partition the network.
 
     Parameters:
     -----------
@@ -369,9 +437,9 @@ def leiden_iterations_java(
 
     # Run the iterations of Leiden
     for i in range(iterations):
-        logger.info("Iteration in progress: {0}".format(i))
+        # logger.info(f"Iteration in progress: {i}")
 
-        output = join(tmp_dir, "partition_{0}.txt".format(i))
+        output = join(tmp_dir, f"partition_{i}.txt")
 
         # Clusterize the network using Leiden.
         cmd = (
@@ -477,7 +545,7 @@ def louvain_iterations_cpp(network_file, iterations, tmp_dir, louvain_path):
 
     # Run the iterations of Louvain
     for i in range(iterations):
-        logger.info("Iteration in progress: {0}".format(i))
+        # logger.info(f"Iteration in progress: {i}")
 
         louvain_args["iteration"] = i
 
@@ -618,7 +686,6 @@ def partition(
     logger.info("Detect overlapping bins:")
     hamming_distance = get_hamming_distance(
         core_bins_iterations,
-        iterations,
         threads,
     )
 
@@ -627,7 +694,6 @@ def partition(
         overlapping_parameter,
         hamming_distance,
         core_bins_contigs,
-        core_bins_iterations,
     )
 
     # Update the contigs_data_file.
@@ -750,7 +816,7 @@ def update_contigs_data(
         core_bin_contigs_number = len(core_bin)
         core_bin_length = sum(core_bin_data.Size)
         # Write the new information
-        contigs_data.loc[core_bin, "Core_bin_ID"] = i + 1
+        contigs_data.loc[core_bin, "Core_bin_ID"] = f"{i + 1:05d}"
         contigs_data.loc[core_bin, "Core_bin_contigs"] = core_bin_contigs_number
         contigs_data.loc[core_bin, "Core_bin_size"] = core_bin_length
 
@@ -762,7 +828,7 @@ def update_contigs_data(
         overlapping_bin_contigs_number = len(overlapping_bin)
         overlapping_bin_length = sum(overlapping_bin_data.Size)
         # Write the new information
-        contigs_data.loc[overlapping_bin, "Overlapping_bin_ID"] = i
+        contigs_data.loc[overlapping_bin, "Overlapping_bin_ID"] = f"{i:05d}"
         contigs_data.loc[
             overlapping_bin, "Overlapping_bin_contigs"
         ] = overlapping_bin_contigs_number
@@ -775,3 +841,39 @@ def update_contigs_data(
     contigs_data.to_csv(contig_data_file_2, sep="\t", header=True, index=False)
 
     return contigs_data, contig_data_file_2
+
+
+# Deprecated Spinglass functions:
+
+
+def spinglass_partition(
+    subnetwork,
+    spins=2,
+):
+    """Use spinglass function from cdlib to partition the network.
+
+    This function is only used in the validation step as it will take a long
+    time to run on a large network.
+
+    Parameters:
+    -----------
+    subnetwork : networkx.classes.graph.Graph
+        Network of interaction of a contaminated bins.
+    spins : int
+        Number of expected MAGs in the contaminated bins.
+
+    Returns:
+    dict:
+        Dictionnary with the id of the contig as key and the clustering result
+        as values.
+    """
+    # Partition the network using spingalss algorithm.
+    # coms = algorithms.spinglass(subnetwork, spins)
+
+    # Extract the clusters.
+    # output_partition = {}
+    # for ids, list_contigs in enumerate(coms.communities):
+    #     for contig in list_contigs:
+    #         output_partition[contig] = str(ids)
+
+    # return output_partition
